@@ -55,6 +55,7 @@
 #define scheme_cancel_sleep (scheme_extension_table->scheme_cancel_sleep)
 #define scheme_start_sleeper_thread (scheme_extension_table->scheme_start_sleeper_thread)
 #define scheme_end_sleeper_thread (scheme_extension_table->scheme_end_sleeper_thread)
+#define scheme_notify_sleep_progress (scheme_extension_table->scheme_notify_sleep_progress)
 #define scheme_make_thread_cell (scheme_extension_table->scheme_make_thread_cell)
 #define scheme_thread_cell_get (scheme_extension_table->scheme_thread_cell_get)
 #define scheme_thread_cell_set (scheme_extension_table->scheme_thread_cell_set)
@@ -64,6 +65,7 @@
 #define scheme_make_custodian (scheme_extension_table->scheme_make_custodian)
 #define scheme_add_managed (scheme_extension_table->scheme_add_managed)
 #define scheme_custodian_check_available (scheme_extension_table->scheme_custodian_check_available)
+#define scheme_custodian_is_available (scheme_extension_table->scheme_custodian_is_available)
 #define scheme_remove_managed (scheme_extension_table->scheme_remove_managed)
 #define scheme_close_managed (scheme_extension_table->scheme_close_managed)
 #define scheme_schedule_custodian_close (scheme_extension_table->scheme_schedule_custodian_close)
@@ -457,6 +459,7 @@
 #define scheme_make_port_type (scheme_extension_table->scheme_make_port_type)
 #define scheme_make_input_port (scheme_extension_table->scheme_make_input_port)
 #define scheme_make_output_port (scheme_extension_table->scheme_make_output_port)
+#define scheme_set_next_port_custodian (scheme_extension_table->scheme_set_next_port_custodian)
 #define scheme_set_port_location_fun (scheme_extension_table->scheme_set_port_location_fun)
 #define scheme_set_port_count_lines_fun (scheme_extension_table->scheme_set_port_count_lines_fun)
 #define scheme_progress_evt_via_get (scheme_extension_table->scheme_progress_evt_via_get)
@@ -620,6 +623,7 @@
 #define scheme_get_milliseconds (scheme_extension_table->scheme_get_milliseconds)
 #define scheme_get_inexact_milliseconds (scheme_extension_table->scheme_get_inexact_milliseconds)
 #define scheme_get_process_milliseconds (scheme_extension_table->scheme_get_process_milliseconds)
+#define scheme_get_thread_milliseconds (scheme_extension_table->scheme_get_thread_milliseconds)
 #define scheme_banner (scheme_extension_table->scheme_banner)
 #define scheme_version (scheme_extension_table->scheme_version)
 #define scheme_check_proc_arity (scheme_extension_table->scheme_check_proc_arity)
