@@ -167,84 +167,88 @@ enum {
   scheme_module_phase_exports_type,     /* 149 */
   scheme_logger_type,                   /* 150 */
   scheme_log_reader_type,               /* 151 */
+  scheme_free_id_info_type,             /* 152 */
+  scheme_rib_delimiter_type,            /* 153 */
+  scheme_noninline_proc_type,           /* 154 */
+  scheme_prune_context_type,            /* 155 */
 
 #ifdef MZTAG_REQUIRED
-  _scheme_last_normal_type_,            /* 152 */
+  _scheme_last_normal_type_,            /* 156 */
 
-  scheme_rt_weak_array,                 /* 153 */
+  scheme_rt_weak_array,                 /* 157 */
 
-  scheme_rt_comp_env,                   /* 154 */
-  scheme_rt_constant_binding,           /* 155 */
-  scheme_rt_resolve_info,               /* 156 */
-  scheme_rt_optimize_info,              /* 157 */
-  scheme_rt_compile_info,               /* 158 */
-  scheme_rt_cont_mark,                  /* 159 */
-  scheme_rt_saved_stack,                /* 160 */
-  scheme_rt_reply_item,                 /* 161 */
-  scheme_rt_closure_info,               /* 162 */
-  scheme_rt_overflow,                   /* 163 */
-  scheme_rt_overflow_jmp,               /* 164 */
-  scheme_rt_meta_cont,                  /* 165 */
-  scheme_rt_dyn_wind_cell,              /* 166 */
-  scheme_rt_dyn_wind_info,              /* 167 */
-  scheme_rt_dyn_wind,                   /* 168 */
-  scheme_rt_dup_check,                  /* 169 */
-  scheme_rt_thread_memory,              /* 170 */
-  scheme_rt_input_file,                 /* 171 */
-  scheme_rt_input_fd,                   /* 172 */
-  scheme_rt_oskit_console_input,        /* 173 */
-  scheme_rt_tested_input_file,          /* 174 */
-  scheme_rt_tested_output_file,         /* 175 */
-  scheme_rt_indexed_string,             /* 176 */
-  scheme_rt_output_file,                /* 177 */
-  scheme_rt_load_handler_data,          /* 178 */
-  scheme_rt_pipe,                       /* 179 */
-  scheme_rt_beos_process,               /* 180 */
-  scheme_rt_system_child,               /* 181 */
-  scheme_rt_tcp,                        /* 182 */
-  scheme_rt_write_data,                 /* 183 */
-  scheme_rt_tcp_select_info,            /* 184 */
-  scheme_rt_namespace_option,           /* 185 */
-  scheme_rt_param_data,                 /* 186 */
-  scheme_rt_will,                       /* 187 */
-  scheme_rt_struct_proc_info,           /* 188 */
-  scheme_rt_linker_name,                /* 189 */
-  scheme_rt_param_map,                  /* 190 */
-  scheme_rt_finalization,               /* 191 */
-  scheme_rt_finalizations,              /* 192 */
-  scheme_rt_cpp_object,                 /* 193 */
-  scheme_rt_cpp_array_object,           /* 194 */
-  scheme_rt_stack_object,               /* 195 */
-  scheme_rt_preallocated_object,        /* 196 */
-  scheme_thread_hop_type,               /* 197 */
-  scheme_rt_srcloc,                     /* 198 */
-  scheme_rt_evt,                        /* 199 */
-  scheme_rt_syncing,                    /* 200 */
-  scheme_rt_comp_prefix,                /* 201 */
-  scheme_rt_user_input,                 /* 202 */
-  scheme_rt_user_output,                /* 203 */
-  scheme_rt_compact_port,               /* 204 */
-  scheme_rt_read_special_dw,            /* 205 */
-  scheme_rt_regwork,                    /* 206 */
-  scheme_rt_buf_holder,                 /* 207 */
-  scheme_rt_parameterization,           /* 208 */
-  scheme_rt_print_params,               /* 209 */
-  scheme_rt_read_params,                /* 210 */
-  scheme_rt_native_code,                /* 211 */
-  scheme_rt_native_code_plus_case,      /* 212 */
-  scheme_rt_jitter_data,                /* 213 */
-  scheme_rt_module_exports,             /* 214 */
-  scheme_rt_delay_load_info,            /* 215 */
-  scheme_rt_marshal_info,               /* 216 */
-  scheme_rt_unmarshal_info,             /* 217 */
-  scheme_rt_runstack,                   /* 218 */
-  scheme_rt_sfs_info,                   /* 219 */
-  scheme_rt_validate_clearing,          /* 220 */
-  scheme_rt_rb_node,                    /* 221 */
+  scheme_rt_comp_env,                   /* 158 */
+  scheme_rt_constant_binding,           /* 159 */
+  scheme_rt_resolve_info,               /* 160 */
+  scheme_rt_optimize_info,              /* 161 */
+  scheme_rt_compile_info,               /* 162 */
+  scheme_rt_cont_mark,                  /* 163 */
+  scheme_rt_saved_stack,                /* 164 */
+  scheme_rt_reply_item,                 /* 165 */
+  scheme_rt_closure_info,               /* 166 */
+  scheme_rt_overflow,                   /* 167 */
+  scheme_rt_overflow_jmp,               /* 168 */
+  scheme_rt_meta_cont,                  /* 169 */
+  scheme_rt_dyn_wind_cell,              /* 170 */
+  scheme_rt_dyn_wind_info,              /* 171 */
+  scheme_rt_dyn_wind,                   /* 172 */
+  scheme_rt_dup_check,                  /* 173 */
+  scheme_rt_thread_memory,              /* 174 */
+  scheme_rt_input_file,                 /* 175 */
+  scheme_rt_input_fd,                   /* 176 */
+  scheme_rt_oskit_console_input,        /* 177 */
+  scheme_rt_tested_input_file,          /* 178 */
+  scheme_rt_tested_output_file,         /* 179 */
+  scheme_rt_indexed_string,             /* 180 */
+  scheme_rt_output_file,                /* 181 */
+  scheme_rt_load_handler_data,          /* 182 */
+  scheme_rt_pipe,                       /* 183 */
+  scheme_rt_beos_process,               /* 184 */
+  scheme_rt_system_child,               /* 185 */
+  scheme_rt_tcp,                        /* 186 */
+  scheme_rt_write_data,                 /* 187 */
+  scheme_rt_tcp_select_info,            /* 188 */
+  scheme_rt_namespace_option,           /* 189 */
+  scheme_rt_param_data,                 /* 190 */
+  scheme_rt_will,                       /* 191 */
+  scheme_rt_struct_proc_info,           /* 192 */
+  scheme_rt_linker_name,                /* 193 */
+  scheme_rt_param_map,                  /* 194 */
+  scheme_rt_finalization,               /* 195 */
+  scheme_rt_finalizations,              /* 196 */
+  scheme_rt_cpp_object,                 /* 197 */
+  scheme_rt_cpp_array_object,           /* 198 */
+  scheme_rt_stack_object,               /* 199 */
+  scheme_rt_preallocated_object,        /* 200 */
+  scheme_thread_hop_type,               /* 201 */
+  scheme_rt_srcloc,                     /* 202 */
+  scheme_rt_evt,                        /* 203 */
+  scheme_rt_syncing,                    /* 204 */
+  scheme_rt_comp_prefix,                /* 205 */
+  scheme_rt_user_input,                 /* 206 */
+  scheme_rt_user_output,                /* 207 */
+  scheme_rt_compact_port,               /* 208 */
+  scheme_rt_read_special_dw,            /* 209 */
+  scheme_rt_regwork,                    /* 210 */
+  scheme_rt_buf_holder,                 /* 211 */
+  scheme_rt_parameterization,           /* 212 */
+  scheme_rt_print_params,               /* 213 */
+  scheme_rt_read_params,                /* 214 */
+  scheme_rt_native_code,                /* 215 */
+  scheme_rt_native_code_plus_case,      /* 216 */
+  scheme_rt_jitter_data,                /* 217 */
+  scheme_rt_module_exports,             /* 218 */
+  scheme_rt_delay_load_info,            /* 219 */
+  scheme_rt_marshal_info,               /* 220 */
+  scheme_rt_unmarshal_info,             /* 221 */
+  scheme_rt_runstack,                   /* 222 */
+  scheme_rt_sfs_info,                   /* 223 */
+  scheme_rt_validate_clearing,          /* 224 */
+  scheme_rt_rb_node,                    /* 225 */
 #endif
 
-  scheme_place_type,                    /* 222 */
-  scheme_engine_type,                   /* 223 */
+  scheme_place_type,                    /* 226 */
+  scheme_engine_type,                   /* 227 */
 
   _scheme_last_type_
 };
