@@ -69,8 +69,10 @@ void scheme_set_case_sensitive(int v) { scheme_case_sensitive =  v; }
 
 /* locals */
 static Scheme_Object *symbol_p_prim (int argc, Scheme_Object *argv[]);
+static Scheme_Object *symbol_interned_p_prim (int argc, Scheme_Object *argv[]);
 static Scheme_Object *string_to_symbol_prim (int argc, Scheme_Object *argv[]);
 static Scheme_Object *string_to_uninterned_symbol_prim (int argc, Scheme_Object *argv[]);
+static Scheme_Object *string_to_unreadable_symbol_prim (int argc, Scheme_Object *argv[]);
 static Scheme_Object *symbol_to_string_prim (int argc, Scheme_Object *argv[]);
 static Scheme_Object *keyword_p_prim (int argc, Scheme_Object *argv[]);
 static Scheme_Object *keyword_lt (int argc, Scheme_Object *argv[]);
@@ -317,8 +319,12 @@ scheme_init_symbol (Scheme_Env *env)
   SCHEME_PRIM_PROC_FLAGS(p) |= SCHEME_PRIM_IS_UNARY_INLINED;
   scheme_add_global_constant("symbol?", p, env);
   
+  p = scheme_make_folding_prim(symbol_interned_p_prim, "symbol-interned?", 1, 1, 1);
+  scheme_add_global_constant("symbol-interned?", p, env);
+  
   GLOBAL_IMMED_PRIM("string->symbol",             string_to_symbol_prim,            1, 1, env);
   GLOBAL_IMMED_PRIM("string->uninterned-symbol",  string_to_uninterned_symbol_prim, 1, 1, env);
+  GLOBAL_IMMED_PRIM("string->unreadable-symbol",  string_to_unreadable_symbol_prim, 1, 1, env);
   GLOBAL_IMMED_PRIM("symbol->string",             symbol_to_string_prim,            1, 1, env);
   GLOBAL_FOLDING_PRIM("keyword?",                 keyword_p_prim,                   1, 1, 1, env);
   GLOBAL_FOLDING_PRIM("keyword<?",                keyword_lt,                       2, -1, 1, env);
@@ -396,19 +402,14 @@ Scheme_Object *
 scheme_intern_exact_symbol_in_table(Scheme_Hash_Table *symbol_table, int kind, const char *name, unsigned int len)
 {
 #if defined(MZ_USE_PLACES) && defined(MZ_PRECISE_GC)
-  mz_proc_thread *self;
-  self = proc_thread_self;
-  if ( scheme_master_proc_thread && scheme_master_proc_thread != proc_thread_self ) {
-    int return_msg_type;
-    void *return_payload;
-    Scheme_Symbol_Parts parts;
-    parts.table = symbol_table;
-    parts.kind = kind;
-    parts.len  = len;
-    parts.name = name;
-    pt_mbox_send_recv(scheme_master_proc_thread->mbox, 3, &parts, self->mbox, &return_msg_type, &return_payload);
-    return (Scheme_Object*) return_payload;
-  }
+  void *return_payload;
+  Scheme_Symbol_Parts parts;
+  parts.table = symbol_table;
+  parts.kind = kind;
+  parts.len  = len;
+  parts.name = name;
+  return_payload = scheme_master_fast_path(3, &parts);
+  return (Scheme_Object*) return_payload;
 #endif
   return scheme_intern_exact_symbol_in_table_worker(symbol_table, kind, name, len);
 }
@@ -667,6 +668,16 @@ symbol_p_prim (int argc, Scheme_Object *argv[])
   return SCHEME_SYMBOLP(argv[0]) ? scheme_true : scheme_false;
 }
 
+static Scheme_Object *
+symbol_interned_p_prim (int argc, Scheme_Object *argv[])
+{
+  if (SCHEME_SYMBOLP(argv[0]))
+    return (SCHEME_SYM_WEIRDP(argv[0]) ? scheme_false : scheme_true);
+  
+  scheme_wrong_type("symbol-interned?", "symbol", 0, argc, argv);
+  return NULL;
+}
+
 static Scheme_Object *
 string_to_symbol_prim (int argc, Scheme_Object *argv[])
 {
@@ -685,6 +696,22 @@ string_to_uninterned_symbol_prim (int argc, Scheme_Object *argv[])
 				       SCHEME_CHAR_STRTAG_VAL(argv[0]));
 }
 
+static Scheme_Object *
+string_to_unreadable_symbol_prim (int argc, Scheme_Object *argv[])
+{
+  char buf[64], *bs;
+  long blen;
+
+  if (!SCHEME_CHAR_STRINGP(argv[0]))
+    scheme_wrong_type("string->symbol", "string", 0, argc, argv);
+
+  bs = scheme_utf8_encode_to_buffer_len(SCHEME_CHAR_STR_VAL(argv[0]),
+                                        SCHEME_CHAR_STRTAG_VAL(argv[0]), 
+                                        buf, 64, &blen);
+
+  return scheme_intern_exact_parallel_symbol(bs, blen);
+}
+
 static Scheme_Object *
 symbol_to_string_prim (int argc, Scheme_Object *argv[])
 {
