@@ -85,7 +85,7 @@ static void init_type_arrays()
 }
 
 void
-scheme_init_type (Scheme_Env *env)
+scheme_init_type ()
 {
   if (!type_names)
     init_type_arrays();
@@ -599,7 +599,7 @@ void scheme_register_traversers(void)
   GC_REG_TRAV(scheme_rt_buf_holder, buf_holder);
   GC_REG_TRAV(scheme_rt_pipe, mark_pipe);
 
-  GC_REG_TRAV(scheme_tcp_accept_evt_type, small_object);
+  GC_REG_TRAV(scheme_tcp_accept_evt_type, twoptr_obj);
 
   GC_REG_TRAV(scheme_special_comment_type, small_object);
 
@@ -619,6 +619,10 @@ void scheme_register_traversers(void)
   GC_REG_TRAV(scheme_log_reader_type, mark_log_reader);
 
   GC_REG_TRAV(scheme_rt_runstack, runstack_val);
+
+  GC_REG_TRAV(scheme_rib_delimiter_type, small_object);
+  GC_REG_TRAV(scheme_noninline_proc_type, small_object);
+  GC_REG_TRAV(scheme_prune_context_type, small_object);
 }
 
 END_XFORM_SKIP;
